pub struct FlagCell<T>(/* private fields */);Expand description
带标记+引用计数+内部可变性的智能容器 逻辑上是唯一所有权持有者,逻辑禁用后可通过FlagRef::resurrect复活
确保在安全使用时,Cell存在即内部数据存在。 正常使用时,逻辑上是不会有人再访问已经释放的数据的,因为确保访问者死完了数据才会释放。
Implementations
Sourceimpl<T> FlagCell<T>
impl<T> FlagCell<T>
Sourcepub fn is_enabled(&self) -> bool
pub fn is_enabled(&self) -> bool
获取数据是否逻辑启用
Sourcepub fn borrow(&self) -> Ref<'_, T>
pub fn borrow(&self) -> Ref<'_, T>
Immutably borrows the wrapped value.
The borrow lasts until the returned Ref exits scope. Multiple
immutable borrows can be taken out at the same time.
Panics
Panics if the value is currently mutably borrowed. For a non-panicking variant, use
try_borrow.
Sourcepub fn borrow_mut(&self) -> RefMut<'_, T>
pub fn borrow_mut(&self) -> RefMut<'_, T>
Mutably borrows the wrapped value.
The borrow lasts until the returned RefMut or all RefMuts derived
from it exit scope. The value cannot be borrowed while this borrow is
active.
Panics
Panics if the value is currently borrowed. For a non-panicking variant, use
try_borrow_mut.
Sourcepub fn try_borrow(&self) -> Option<Ref<'_, T>>
pub fn try_borrow(&self) -> Option<Ref<'_, T>>
Immutably borrows the wrapped value, returning an error if the value is currently mutably borrowed.
The borrow lasts until the returned Ref exits scope. Multiple immutable borrows can be
taken out at the same time.
This is the non-panicking variant of borrow.
Sourcepub fn try_borrow_mut(&self) -> Option<RefMut<'_, T>>
pub fn try_borrow_mut(&self) -> Option<RefMut<'_, T>>
Mutably borrows the wrapped value, returning an error if the value is currently borrowed.
The borrow lasts until the returned RefMut or all RefMuts derived
from it exit scope. The value cannot be borrowed while this borrow is
active.
This is the non-panicking variant of borrow_mut.
Sourcepub fn as_ref_cell_ref(&self) -> &RefCell<ManuallyDrop<T>>
pub fn as_ref_cell_ref(&self) -> &RefCell<ManuallyDrop<T>>
得到内部RefCell的引用
Sourcepub fn as_ref_cell_ptr(&self) -> *const RefCell<ManuallyDrop<T>>
pub fn as_ref_cell_ptr(&self) -> *const RefCell<ManuallyDrop<T>>
得到内部RefCell的指针
Sourcepub fn flag_borrow(&self) -> FlagRef<T>
pub fn flag_borrow(&self) -> FlagRef<T>
生成一个 FlagRef
Sourcepub fn replace(&self, value: T) -> T
pub fn replace(&self, value: T) -> T
Replaces the wrapped value with a new one, returning the old value, without deinitializing either one.
This function corresponds to mem::replace.
Panics
Panics if the value is currently borrowed.
For non-panicking variant , see try_replace.
Sourcepub fn try_replace(&self, value: T) -> Result<T, T>
pub fn try_replace(&self, value: T) -> Result<T, T>
Replaces the wrapped value with a new one, returning the old value, without deinitializing either one.
This function corresponds to mem::replace.
如果当前存在引用,返回Err返还传入值
This is the non-panicking variant of replace.
Sourcepub fn unwrap(self) -> T
pub fn unwrap(self) -> T
消费自身,返回内部数据,同时禁用
Panics
若当前存在任何引用(包括FlagRef),或被异常禁用,panic。
For non-panicking variant , see try_unwrap.
Sourcepub fn try_unwrap(self) -> Result<T, Self>
pub fn try_unwrap(self) -> Result<T, Self>
Methods from Deref<Target = RefCell<ManuallyDrop<T>>>
1.24.0 · Sourcepub fn replace(&self, t: T) -> T
pub fn replace(&self, t: T) -> T
Replaces the wrapped value with a new one, returning the old value, without deinitializing either one.
This function corresponds to std::mem::replace.
Panics
Panics if the value is currently borrowed.
Examples
use std::cell::RefCell;
let cell = RefCell::new(5);
let old_value = cell.replace(6);
assert_eq!(old_value, 5);
assert_eq!(cell, RefCell::new(6));1.35.0 · Sourcepub fn replace_with<F>(&self, f: F) -> T
pub fn replace_with<F>(&self, f: F) -> T
Replaces the wrapped value with a new one computed from f, returning
the old value, without deinitializing either one.
Panics
Panics if the value is currently borrowed.
Examples
use std::cell::RefCell;
let cell = RefCell::new(5);
let old_value = cell.replace_with(|&mut old| old + 1);
assert_eq!(old_value, 5);
assert_eq!(cell, RefCell::new(6));1.24.0 · Sourcepub fn swap(&self, other: &RefCell<T>)
pub fn swap(&self, other: &RefCell<T>)
Swaps the wrapped value of self with the wrapped value of other,
without deinitializing either one.
This function corresponds to std::mem::swap.
Panics
Panics if the value in either RefCell is currently borrowed, or
if self and other point to the same RefCell.
Examples
use std::cell::RefCell;
let c = RefCell::new(5);
let d = RefCell::new(6);
c.swap(&d);
assert_eq!(c, RefCell::new(6));
assert_eq!(d, RefCell::new(5));1.0.0 · Sourcepub fn borrow(&self) -> Ref<'_, T>
pub fn borrow(&self) -> Ref<'_, T>
Immutably borrows the wrapped value.
The borrow lasts until the returned Ref exits scope. Multiple
immutable borrows can be taken out at the same time.
Panics
Panics if the value is currently mutably borrowed. For a non-panicking variant, use
try_borrow.
Examples
use std::cell::RefCell;
let c = RefCell::new(5);
let borrowed_five = c.borrow();
let borrowed_five2 = c.borrow();An example of panic:
use std::cell::RefCell;
let c = RefCell::new(5);
let m = c.borrow_mut();
let b = c.borrow(); // this causes a panic1.13.0 · Sourcepub fn try_borrow(&self) -> Result<Ref<'_, T>, BorrowError>
pub fn try_borrow(&self) -> Result<Ref<'_, T>, BorrowError>
Immutably borrows the wrapped value, returning an error if the value is currently mutably borrowed.
The borrow lasts until the returned Ref exits scope. Multiple immutable borrows can be
taken out at the same time.
This is the non-panicking variant of borrow.
Examples
use std::cell::RefCell;
let c = RefCell::new(5);
{
let m = c.borrow_mut();
assert!(c.try_borrow().is_err());
}
{
let m = c.borrow();
assert!(c.try_borrow().is_ok());
}1.0.0 · Sourcepub fn borrow_mut(&self) -> RefMut<'_, T>
pub fn borrow_mut(&self) -> RefMut<'_, T>
Mutably borrows the wrapped value.
The borrow lasts until the returned RefMut or all RefMuts derived
from it exit scope. The value cannot be borrowed while this borrow is
active.
Panics
Panics if the value is currently borrowed. For a non-panicking variant, use
try_borrow_mut.
Examples
use std::cell::RefCell;
let c = RefCell::new("hello".to_owned());
*c.borrow_mut() = "bonjour".to_owned();
assert_eq!(&*c.borrow(), "bonjour");An example of panic:
use std::cell::RefCell;
let c = RefCell::new(5);
let m = c.borrow();
let b = c.borrow_mut(); // this causes a panic1.13.0 · Sourcepub fn try_borrow_mut(&self) -> Result<RefMut<'_, T>, BorrowMutError>
pub fn try_borrow_mut(&self) -> Result<RefMut<'_, T>, BorrowMutError>
Mutably borrows the wrapped value, returning an error if the value is currently borrowed.
The borrow lasts until the returned RefMut or all RefMuts derived
from it exit scope. The value cannot be borrowed while this borrow is
active.
This is the non-panicking variant of borrow_mut.
Examples
use std::cell::RefCell;
let c = RefCell::new(5);
{
let m = c.borrow();
assert!(c.try_borrow_mut().is_err());
}
assert!(c.try_borrow_mut().is_ok());1.12.0 · Sourcepub fn as_ptr(&self) -> *mut T
pub fn as_ptr(&self) -> *mut T
Returns a raw pointer to the underlying data in this cell.
Examples
use std::cell::RefCell;
let c = RefCell::new(5);
let ptr = c.as_ptr();1.37.0 · Sourcepub unsafe fn try_borrow_unguarded(&self) -> Result<&T, BorrowError>
pub unsafe fn try_borrow_unguarded(&self) -> Result<&T, BorrowError>
Immutably borrows the wrapped value, returning an error if the value is currently mutably borrowed.
Safety
Unlike RefCell::borrow, this method is unsafe because it does not
return a Ref, thus leaving the borrow flag untouched. Mutably
borrowing the RefCell while the reference returned by this method
is alive is undefined behavior.
Examples
use std::cell::RefCell;
let c = RefCell::new(5);
{
let m = c.borrow_mut();
assert!(unsafe { c.try_borrow_unguarded() }.is_err());
}
{
let m = c.borrow();
assert!(unsafe { c.try_borrow_unguarded() }.is_ok());
}